package body STM32.GPIO.Ports is

	package body GPIO_Port_Boolean is

		procedure Set (Value : Boolean) is
		begin
			Register.BSRR := (
				BR => 2**Bit,
				BS => 2**Bit * Boolean'Pos(Value xor Invert)
			);
		end;

		function Value return Boolean is
		begin
			return (Register.IDR and 2**Bit) /= 0;
		end;

		procedure Set_MODER (Mode : Port_Mode) is
		begin
			Register.MODER (Bit) := Mode;
		end;

		procedure Set_OTYPER (Output : Output_Type) is
		begin
			Register.OTYPER (Bit) := Output;
		end;

		procedure Set_OSPEEDR (Speed : Output_Speed) is
		begin
			Register.OSPEEDR (Bit) := Speed;
		end;

		procedure Set_PUPDR (Pull : Port_Pull) is
		begin
			Register.PUPDR (Bit) := Pull;
		end;

		procedure Set_AFR (Value : Alternate_Function) is
		begin
			Register.AFR (Bit) := Value;
		end;

	end GPIO_Port_Boolean;

	package body GPIO_Port_Modular is

		Size : constant Positive := 1 + Last_Bit - First_Bit;
		Mask : constant Unsigned_16 := 2**(Last_Bit + 1) - 2**First_Bit;

		procedure Set(Value : Value_Type) is
		begin
			Register.BSRR := (
				BR => Mask,
				BS => 2**First_Bit * Unsigned_16(Value xor Invert)
			);
		end;

		function Value return Value_Type is
		begin
			return Value_Type((Register.IDR / 2**First_Bit) and (2**Size - 1)) xor Invert;
		end;

		procedure Set_MODER (Mode : Port_Mode) is
			R : Port_Mode_Register := Register.MODER;
		begin
			for I in Port_Bit_Number range First_Bit .. Last_Bit loop
				R (I) := Mode;
			end loop;

			Register.MODER := R;
		end;

		procedure Set_OTYPER (Output : Output_Type) is
			R : Output_Type_Register := Register.OTYPER;
		begin
			for I in Port_Bit_Number range First_Bit .. Last_Bit loop
				R (I) := Output;
			end loop;

			Register.OTYPER := R;
		end;

		procedure Set_OSPEEDR (Speed : Output_Speed) is
			R : Output_Speed_Register := Register.OSPEEDR;
		begin
			for I in Port_Bit_Number range First_Bit .. Last_Bit loop
				R (I) := Speed;
			end loop;

			Register.OSPEEDR := R;
		end;

		procedure Set_PUPDR (Pull : Port_Pull) is
			R : Port_Pull_Register := Register.PUPDR;
		begin
			for I in Port_Bit_Number range First_Bit .. Last_Bit loop
				R (I) := Pull;
			end loop;

			Register.PUPDR := R;
		end;

		procedure Set_AFR (Value : Alternate_Function) is
			R : Alternate_Function_Register := Register.AFR;
		begin
			for I in Port_Bit_Number range First_Bit .. Last_Bit loop
				R (I) := Value;
			end loop;

			Register.AFR := R;
		end;
	end GPIO_Port_Modular;

end STM32.GPIO.Ports;
